<?php
/**
 * @package         SourceCoast Extensions
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\Registry\Registry;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Filesystem\Folder;

jimport('sourcecoast.plugins.jfbconnectPlugin');
jimport('joomla.application.component.controller');
jimport('joomla.filesystem.file');

class SocialProfilePlugin extends JFBConnectPlugin
{
    var $defaultSettings;
    var $settings;
    var $_componentFile = '';
    var $_componentFolder = '';
    var $network;
    var $profileLibrary;
    var $joomlaId;
    var $socialId;
    var $registrationUser; // Original User object used for registration. Mainly necessary so the password_clear can be read by the plugins

    var $db;
    var $app;
    var $_importEnabled = false; // Can this plugin import previous FB connections
    private $componentLoaded = null;

    protected $profileName;

    // Set this to a non-null value to allow 3rd party registration flows
    protected $registration_url = null;

    var $_settingShowRegistrationFields = null;
    var $_settingShowImportedFields = null;

    function __construct(&$subject, $params)
    {
        $this->profileName = $params['name'];
        $this->db = Factory::getDBO();
        $this->app = Factory::getApplication();
        $this->defaultSettings = new Registry();

        parent::__construct($subject, $params);
    }

    function __get($name)
    {
        switch ($name)
        {
            case 'registration_url' :
                return $this->registration_url;
            case 'name' :
                return $this->profileName;
            case 'displayName' :
            {
                if (isset($this->displayName))
                    return $this->displayName;
                else
                    return ucwords($this->profileName);
            }
        }
    }

    public function onSocialProfilesGetPlugins()
    {
        if($this->isLoaded())
            return $this;
        else
            return null;
    }

    /**
     * Called after registration occurs
     * Good for importing the profile on first registration
     */
    public function onSocialProfilesOnRegister($network, $jUser, $socialId)
    {
        if($this->isLoaded())
        {
            $this->registrationUser = $jUser;
            $this->loadSettings($network, $jUser->get('id'), $socialId);
            $this->onRegister();

            return true;
        }
        return false;
    }

    /**
     * OnRegister
     * Will call (optional) functions in the profile plugin to create the user and log them in for the first time
     *
     * @param $joomlaId
     * @param $fbUserId
     */
    protected function onRegister()
    {
        if (empty($this->joomlaId) || empty($this->socialId))
            return; // Something's wrong, we should have both of these

        $profileData = $this->fetchProfileFromFieldMap(true);

        // Create the new user in the 3rd party extension with profile information imported or input on the registration page
        $this->createUser($profileData);

        if ($this->settings->get('import_avatar'))
            $this->importSocialAvatar();

        if ($this->settings->get('import_status'))
            $this->importSocialStatus();

        if ($this->settings->get('import_cover_photo', 0))
            $this->importCoverPhoto();
    }

    /**
     * Called after registration occurs
     * Good for importing the profile on first registration
     */
    function onSocialProfilesOnLogin($network, $joomlaId, $socialId)
    {
        if($this->isLoaded())
        {
            $this->loadSettings($network, $joomlaId, $socialId);
            $this->onLogin();

            return true;
        }
        return false;
    }

    protected function onLogin()
    {
        if (empty($this->joomlaId) || empty($this->socialId))
            return; // Something's wrong, we should have both of these

        if ($this->settings->get('import_always'))
        {
            $this->importSocialProfile(); // This will import fields from the user's profile

            if ($this->settings->get('import_avatar'))
                $this->importSocialAvatar();

            if ($this->settings->get('import_status'))
                $this->importSocialStatus();

            if ($this->settings->get('import_cover_photo', 0))
                $this->importCoverPhoto();
        }
    }

    /*
     * Called on user request or other times like mapping
     */
    public function socialProfilesOnImportProfile($network, $joomlaId, $socialId)
    {
        if($this->isLoaded())
        {
            $this->loadSettings($network, $joomlaId, $socialId);
            if (empty($this->joomlaId) || empty($this->socialId))
                return; // Something's wrong, we should have both of these

            if ($this->getProfileImportPermission())
            {
                $this->importSocialProfile(); // This will import fields from the user's profile

                if ($this->settings->get('import_avatar'))
                    $this->importSocialAvatar();

                if ($this->settings->get('import_status'))
                    $this->importSocialStatus();
            }
        }
    }

    public function onSocialProfilesGetRequiredScope($network)
    {
        $scope = array();

        if($this->isLoaded())
        {
            $this->loadSettings($network);
            $fieldMap = $this->getFieldMap($network);
            $scope = array_merge($scope, $this->profileLibrary->getPermissionsForFields($fieldMap));
        }
        return $scope;
    }

    public function onSocialProfilesOnNewUserSave($network, $joomlaId, $socialId)
    {
        if($this->isLoaded())
        {
            $regComponent = JFBCFactory::config()->get('registration_component');
            if ($regComponent == $this->name)
            {
                $this->loadSettings($network, $joomlaId, $socialId);
                $this->onNewUserSave();
            }
        }
    }

    protected function onNewUserSave()
    {
        return true;
    }

    protected function createUser($profileData)
    {

    }

    /**
     * Profile plugin (or integration component) will send the new user emails.
     * In this case, the JFBConnect/JLinked/etc will not send the admin/user emails
     * @return bool
     */
    public function onSocialProfilesSendsNewUserEmails()
    {
        return false;
    }

    /**
     * Profile will add its form validation script. If no custom validation is required,
     * default validation will be performed
     * @return bool
     */
    public function onSocialProfilesAddFormValidation()
    {
        return false;
    }


    protected function importSocialStatus()
    {
        $socialStatus = $this->profileLibrary->fetchStatus($this->socialId);
        if (!empty($socialStatus))
            $this->setStatus($socialStatus);
    }

    // Deprecated - use $this->name instead
    function getName()
    {
        return $this->profileName;
    }

    public function onSocialProfilesPrefillRegistration()
    {
        if($this->isLoaded())
        {
            $regComponent = JFBCFactory::config()->get('registration_component');
            if ($regComponent != $this->name)
                return;

            $provider = $this->app->getUserState('com_jfbconnect.registration.provider.name');
            $providerUserId = $this->app->getUserState('com_jfbconnect.registration.provider.user_id');

            if ($provider && $providerUserId)
            {
                $this->loadSettings($provider, null, $providerUserId);
                if ($this->prefillRegistration())
                {
                    if (JFBCFactory::config()->get('joomla_skip_newuser_activation') == 1)
                    {
                        $params = ComponentHelper::getParams('com_users');
                        $params->set('useractivation', 0);
                    }
                }
            }
        }
    }

    protected function finalizeRegistration()
    {
        // Try to auto-login the user if that will work
        $params = ComponentHelper::getParams('com_users');
        if (JFBCFactory::config()->get('joomla_skip_newuser_activation') == 1 || !$params->get('useractivation'))
        {
            $provider = $this->app->getUserState('com_jfbconnect.registration.provider.name');
            $redirect = 'index.php?option=com_jfbconnect&task=login.login&provider=' . $provider;
            $this->app->redirect($redirect);
        }
        else
            $this->app->setUserState('com_jfbconnect.registration.alternateflow', false);

        return true;
    }

    protected function prefillRegistration()
    {
        return;
    }

    // Method to set a registration field during registration. Will check if field is already set (likely by the user) and not overwrite that value.
    protected function prefillRegistrationField($fieldName, $value, $method = "POST")
    {
        if (!$this->app->input->get($fieldName, null, $method))
            $this->app->input->set($fieldName, $value, $method);
    }

    /**
     * Get field names and inputs to request additional information from users on registration
     * @return string HTML of form fields to display to user on registration
     */
    public function onSocialProfilesOnShowRegisterForm($network)
    {
        $html = '';
        if($this->isLoaded())
        {
            $this->loadSettings($network);
            $profileData = $this->fetchProfileFromFieldMap(false);
            $html = $this->getRegistrationForm($profileData);
        }
        return $html;
    }

    protected function getRegistrationForm($profileData)
    {
        $showRegistrationFields = $this->settings->get('registration_show_fields');
        $showImportedFields = $this->settings->get('imported_show_fields');

        $html = "";

        $profileFields = $this->getProfileFields();

        $fieldMap = $this->getFieldMap($this->network);
        foreach ($profileFields as $profileField)
        {
            if (property_exists($fieldMap, $profileField->id))
            {
                $id = $profileField->id;
                $fieldName = $fieldMap->$id;
            }
            else
                $fieldName = 0;
            $showField = $showRegistrationFields == "1" &&
                ($showImportedFields == "1" || ($showImportedFields == "0" && !$fieldName));

            if (!$showField)
            {
                if ($fieldName != '0')
                    $this->set('performsSilentImport', 1);
                continue;
            }

            $fieldValue = $profileData->getFieldWithUserState($profileField->id);

            $html .= '<label for="' . $profileField->id . '">' . $profileField->name . '</label>';
            $html .= '<input type="text" name="' . $profileField->id . '" id="' . $profileField->id . '" value="' . $fieldValue . '" /><br/>';
        }

        return $html;
    }

    /*
     * Fetches the user profile from the social network based on the field mapping settings
     * Then, merges the data with any POSTed fields from a possible registration submission
     *
     * Social network profile is saved to the session on the first time to avoid fetching it on every submission or after successful registration
     * @return Registry with profile data
     */
    protected function fetchProfileFromFieldMap($permissionNeeded = false)
    {
        $fieldMap = $this->getFieldMap($this->network);

        $sessionKey = 'plg_socialprofiles.' . $this->network . '.' . $this->name;
        if ($permissionNeeded)
            $permissionGranted = $this->getProfileImportPermission();
        else
            $permissionGranted = true;
        $socialProfile = $this->profileLibrary->fetchProfileFromFieldMap($fieldMap, $permissionGranted);
        $this->app->setUserState($sessionKey, $socialProfile->toString());

        return $socialProfile;
    }

    /**
     * Used for plugins to check any credentials or information as necessary
     * Return true if login should proceed, false if not
     */
    public function onSocialProfilesOnAuthenticate($network, $joomlaId, $socialId)
    {
        if($this->isLoaded())
        {
            $this->loadSettings($network, $joomlaId, $socialId);
            return $this->onAuthenticate();
        }
        return null;
    }

    protected function onAuthenticate()
    {
        $response = new profileResponse();
        $response->status = true;
        return $response;
    }

    /**
     * Determine if the Login Register view needs to give user the option to approve profile import
     * Required for LinkedIn, which requires explicit permission to import the users profile
     * @return bool if permission is needed, false if not
     */
    public function socialProfilesNeedsImportPermission($network)
    {
        return false;
    }

    public function socialProfilesAwardPoints($name, $data)
    {
        if($this->isLoaded())
        {
            $userId = $data->get('userId', null);
            if (!$userId)
                $userId = Factory::getUser()->get('id');
            if (!$userId)
                $userId = $this->joomlaId;

            if ($userId)
                $this->awardPoints($userId, $name, $data);
        }
    }

    protected function awardPoints($userId, $name, $data)
    {
    }

    /*     * *
     *
     * ************ END Triggered functions ************
     *
     * ** */

    /*     * *
     * ************ Direct call functions **************
     */


    /*     * ***
     * ************* END Direct call functions ********8
     */

    // These functions should be overridden by the plugins
    protected function getProfileFields()
    {
        return array();
    }

    protected function getProfileImportPermission()
    {
        return true;
    }

    protected function setAvatar($socialAvatar)
    {
        return true;
    }

    protected function getFieldMap($network)
    {
        $data = $this->settings->get('field_map.' . $network);

        if (!is_object($data))
            $data = new stdClass();
        return $data;
    }

    protected function importSocialProfile()
    {
        $fieldMap = $this->getFieldMap($this->network);
        $socialProfile = $this->fetchProfileFromFieldMap(true);
        if ($socialProfile)
        {
            foreach ($fieldMap as $fieldId => $socialField)
            {
                // Fetch the value from the POST data (if present) or the imported data from the Social Network
                $value = $socialProfile->getFieldWithUserState($fieldId);

                if ($value != null && $value != "")
                {
                    if (is_array($value))
                    { // This is a field with multiple, comma separated values
                        // Remove empty values to prevent blah, , blah as output
                        unset($value['id']); // Remove id key which is useless to import
                        $value = JFBConnectUtilities::r_implode(', ', $value);
                    }
                    $this->saveProfileField($fieldId, $value);
                }
            }
        }
    }

    protected function saveProfileField($fieldId, $value)
    {
        return true;
    }

    protected function importSocialAvatar()
    {
        jimport('joomla.filesystem.file');
        jimport('joomla.utilities.utility');

        $avatarURL = $this->profileLibrary->getAvatarURL($this->socialId, true);
        if ($avatarURL == null)
        {
            if ($this->registrationUser) // Only revert to the default avatar if new user registration
                $this->setDefaultAvatar($this->joomlaId);
            return false;
        }

        $data = JFBConnectUtilities::getRemoteContent($avatarURL);
        if ($data)
        {
            $baseImgPath = $this->getAvatarPath();
            $tmpImgName = 'scprofile_' . $this->joomlaId . '_pic_tmp.jpg';
            $path = $baseImgPath . '/' . $tmpImgName;
            File::write($path, $data);

            // Convert to jpg
            list($width, $height, $type) = getimagesize($path);
            if ($type != 2)
            {
                $image_p = imagecreatetruecolor($width, $height);
                $origImage = imagecreatefromstring($data);
                imagecopyresampled($image_p, $origImage, 0, 0, 0, 0, $width, $height, $width, $height);
                if (!imagejpeg($image_p, $path, 100))
                    return false;
            }

            if ($this->setAvatar($tmpImgName))
                return true;
        }

        # there was a problem adding the avatar, use the default
        $this->setDefaultAvatar();
        return false;

    }

    protected function importCoverPhoto()
    {
        $cover = $this->profileLibrary->getCoverPhoto($this->socialId);
        if (!$cover)
            return false;

        $data = JFBConnectUtilities::getRemoteContent($cover->get('url'));
        $tmpImgName = 'sccover_' . $this->network . $this->socialId . '_pic_tmp.jpg';
        File::write($this->getAvatarPath() . '/' . $tmpImgName, $data);
        $cover->set('path', $this->getAvatarPath() . '/' . $tmpImgName);
        list($width, $height, $type) = getimagesize($cover->get('path'));

        // Resize to the offsets provided if non-0
        if ($type != 2 || $cover->get('offsetX') != 0 || $cover->get('offsetY') != 0)
        {
            $image_p = imagecreatetruecolor($width - $cover->get('offsetX'), $height - $cover->get('offsetY'));
            $origImage = imagecreatefromstring($data);
            imagecopyresampled($image_p, $origImage, 0, 0, $cover->get('offsetX'), $cover->get('offsetY'), $width, $height, $width, $height);
            if (!imagejpeg($image_p, $this->getAvatarPath() . '/' . $tmpImgName, 100))
                return false;
        }
        $cover->set('type', "image/jpeg");
        return $this->setCoverPhoto($cover);
    }

    protected function setCoverPhoto($cover)
    {
        return true;
    }

    protected function getAvatarPath()
    {
        $tmpPath = $this->app->getCfg('tmp_path');
        return $tmpPath;
    }

    protected function setDefaultAvatar()
    {
        return true;
    }

    public function canImportConnections()
    {
        return $this->_importEnabled;
    }

    protected function isLoaded()
    {
        if ($this->componentLoaded === null)
        {
            $this->componentLoaded = true;

            if ($this->_componentFile != '')
            {
                jimport('joomla.filesystem.file');
                $this->componentLoaded = File::exists($this->_componentFolder . '/' . $this->_componentFile);
            }
            else if ($this->_componentFolder != '')
            {
                jimport('joomla.filesystem.folder');
                $this->componentLoaded = Folder::exists($this->_componentFolder);
            }
        }

        return $this->componentLoaded && parent::isLoaded();
    }

    protected function getAutoUsername($providerProfile)
    {
        $usernamePrefixFormat = JFBCFactory::config()->get('auto_username_format');
        $provider = $this->network;
        $providerPrefix = "__";
        if ($provider == 'facebook')
            $providerPrefix = "fb_";
        else if ($provider == "google")
            $providerPrefix = "g_";
        else if ($provider == "twitter")
            $providerPrefix = "t_";

        return JFBConnectUtilities::getAutoUsername($providerProfile->get('first_name'), $providerProfile->get('last_name'), $providerProfile->get('email'), $providerPrefix, $this->socialId, $usernamePrefixFormat);
    }

    protected function loadSettings($network, $joomlaId = null, $socialId = null)
    {
        $this->joomlaId = $joomlaId;
        $this->socialId = $socialId;
        $this->profileLibrary = JFBCFactory::provider($network)->profile;
        $this->network = $network;

        if ($this->settings)
            return $this->settings;

        $query = $this->db->getQuery(true);
        $query->select("value");
        $query->where('setting="profile_' . $this->name . '"');
        $query->from("#__jfbconnect_config");

        $this->db->setQuery($query);
        $values = $this->db->loadResult();
        if (!empty($values))
        {
            $this->settings = new Registry();
            $this->settings->loadString($values);
        }
        else
            $this->settings = $this->defaultSettings;

        return $this->settings;
    }

    public function getFieldMappingHTML()
    {
        $allProviders = JFBCFactory::getAllProviders();
        // Remove providers who aren't configured for authentication
        $providers = $allProviders;
        for ($i = 0; $i < count($allProviders); $i++)
        {
            $p = $allProviders[$i];
            if (!$p->appId)
                unset($providers[$i]);
        }

        $profileFields = $this->getProfileFields();
        $html = '<div class="row"><div class="col-md-12">
            <div class="options-form card">
    <legend>Social Network Profile Import Configuration</legend>
    <table>
        <tr>
            <th>' . ucwords($this->profileName) . " Field</th>";
        foreach ($providers as $provider)
            $html .= "<th>" . $provider->name . "</th>";

        $html .= "</tr>";
        foreach ($profileFields as $profileField)
        {
            $html .= '<tr><td>';
            $html .= Text::_($profileField->name);
            foreach ($providers as $provider)
            {
                $this->loadSettings($provider->systemName);
                $fieldMap = $this->getFieldMap($provider->systemName);
                if (property_exists($fieldMap, $profileField->id))
                {
                    $fieldId = $profileField->id;
                    $selectedValue = $fieldMap->$fieldId;
                }
                else
                    $selectedValue = "0";
                $html .= '</td><td>';
                $profile = $provider->profile;
                $html .= '<select class="form-select" name="profiles_' . $this->profileName . '_' . $provider->systemName . '_field_map' . $profileField->id . '">';
                foreach ($profile->getProviderFields() as $name => $providerField)
                {
                    if ((string) $name == $selectedValue)
                        $selected = 'selected';
                    else
                        $selected = '';

                    if (property_exists($providerField, 'title'))
                        $title = $providerField->title;
                    else
                        $title = $providerField;
                    $html .= '<option value="' . $name . '" ' . $selected . '>' . $title . '</option>';
                }
                $html .= '</select>';
            }
            $html .= '</td></tr>';
        }
        $html .= "</table>";
        $html .= "</div></div></div>";
        return $html;
    }
}

if (!class_exists('profileResponse'))
{
    class profileResponse
    {

        var $status;
        var $message;

    }
}