<?php
/**
 * @package         JFBConnect
 * @copyright (c)   2009-2021 by SourceCoast - All Rights Reserved
 * @license         http://www.gnu.org/licenses/gpl-2.0.html GNU/GPL
 * @version         Release v9.0.197
 * @build-date      2022/02/01
 */

// Check to ensure this file is included in Joomla!
if (!(defined('_JEXEC') || defined('ABSPATH'))) {     die('Restricted access'); };

use Joomla\CMS\Language\Text;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

include_once(JPATH_ADMINISTRATOR . '/components/com_jfbconnect/assets/facebook-api/facebook.php');

class JFBConnectProviderFacebookbusiness extends JFBConnectProvider
{
    // Override model's getInstance to really only get the instance
    private $apiError;

    function __construct()
    {
        $this->name = 'Facebook Business';
        $this->systemName = 'facebookbusiness';
        $this->usernamePrefix = "fbbiz_";
        $this->loginPrefix = "fbbiz";

        // Don't show this normally. Only used for channels
        $this->showLoginOnFrontEnd = false;

        parent::__construct();

        $this->client = new JFBConnectProviderFacebookbusinessClient(array(
                'providerSystemName' => $this->systemName,
                'appId' => $this->appId,
                'secret' => $this->secretKey,
                'cookie' => true)
        );
    }

    function api($api, $params = null, $callAsUser = true, $method = null, $suppressErrors = false)
    {
        $this->apiError = null;
        if (!$method) {
            if ($params)
                $method = "POST";
            else
                $method = "GET";
        }

        if (!$callAsUser)
            $params['access_token'] = $this->appId . "|" . $this->secretKey;
        /*        else if (!$params || (is_array($params) && (!array_key_exists('access_token', $params))))
        {
            // Get the access token for the current user
            $jUser = Factory::getUser();
            $userMapModel = new JFBConnectModelUserMap();
            $userMapModel->getData($jUser->get('id'));
            $accessToken = $userMapModel->_data->access_token;
            if ($accessToken != '' && $accessToken != null)
                $params['access_token'] = $accessToken;
        }*/

        try {
            if ($params != null) // Graph API call with paramters (either App call or POST call)
                $apiData = $this->client->api($api, $method, $params);
            else // Graph API call to only get data
                $apiData = $this->client->api($api);
        } catch (JFBCFacebookApiException $e) {
            $this->apiError = $e->getMessage();
            // Only display errors on the front-end if the config is set to do so
            $app = Factory::getApplication();
            if (!$suppressErrors && ($app->isClient('administrator') || $this->configModel->get('facebook_display_errors'))) {
                JFBCFactory::log(Text::_('COM_JFBCONNECT_FB_API_ERROR') . $e->getMessage(), 'error');
            }
            $apiData = null;
        }

        return $apiData;
    }

    public function getLastError()
    {
        $return = $this->apiError;
        $this->apiError = null;
        return $return;
    }

    function getLogoutButton()
    {
        JFBConnectUtilities::loadLanguage('com_jfbconnect');
        $logoutStr = Text::_('COM_JFBCONNECT_LOGOUT');

        return '<input type="submit" name="Submit" id="jfbcLogoutButton" class="button btn btn-primary" value="' . $logoutStr . '" onclick="javascript:jfbc.login.logout_button_click()" />';
    }

    /* getProviderUserId
    * The new way to get the userId from Facebook as of v5.1. This is the ID from Facebook even if the user is not logged into Joomla.
    */
    function getProviderUserId()
    {
        return $this->client->getUser();
    }

    public function userIsConnected()
    {
        static $connected = null;
        if ($connected == null)
            $connected = $this->getMappedUserId() == null ? false : true;

        return $connected;
    }

    public function onBeforeLogin()
    {
        $this->client->setExtendedAccessToken();
        $this->client->getUser();

        // Ensure the cookie is deleted from the correct path
        setcookie('fbsr_' . $this->appId, '', time() - 1, '/');

        parent::onBeforeLogin();
    }

    // Check that additional scope for a user exists. If not, reroute them through Facebook login to obtain it
    public function fetchNewScope($newScope)
    {
        $params = array();
        $params['scope'] = implode(',', $newScope);
        $params['redirect_uri'] = Uri::base() . 'index.php?option=com_jfbconnect&task=authenticate.callback&provider=facebookbusiness';
        $redirect = $this->client->getLoginUrl($params);
        Factory::getApplication()->redirect($redirect);
    }

    /***
     * Return an array of valid scope
     *
     * $uid = Facebook user ID to look at scope for
     * $token = If provided, will be the token used to request scope.
     * If no token provided, the token stored in the DB will be used
     */
    public function getUserScope($uid, $token = null)
    {
        // get current scope for the user
        $return = array();
        $jid = JFBCFactory::usermap()->getJoomlaUserId($uid, 'facebookbusiness');
        if ($token)
            $params['access_token'] = $token;
        else
            $params['access_token'] = JFBCFactory::usermap()->getUserAccessToken($jid, 'facebookbusiness');
        $currentScope = $this->api('/' . $uid . '/permissions', $params, true, 'GET');
        if (isset($currentScope['data']) && isset($currentScope['data'][0])) {
            foreach ($currentScope['data'] as $scope) {
                // Check for v2.0 of Graph API
                if (array_key_exists('permission', $scope)) {
                    if ($scope['status'] == 'granted')
                        $return[] = $scope['permission'];
                } else {
                    foreach ($scope as $permission => $val) {
                        if ($val == 1)
                            $return[] = $permission;
                    }
                }
            }
        }
        return $return;
    }

    /**
     * Get an array of custom set scopes by the administrator
     * @return array
     */
//    public function getCustomScopes()
//    {
//        $customScopes = array();
//        $customPermsSetting = JFBCFactory::config()->get('facebook_perm_custom');
//        if ($customPermsSetting != '')
//        {
//            $customPermsSetting = str_replace("\r\n", ',', $customPermsSetting);
//            //Separate into an array to be able to merge and then take out duplicates
//            $customPerms = explode(',', $customPermsSetting);
//            foreach ($customPerms as $customPerm)
//                $customScopes[] = trim($customPerm);
//        }
//        return $customScopes;
//    }

    /**
     * Get a comma-separated list of scopes to request from a normal user on login
     * @return string
     */
    public function getRequestScopeString()
    {
        $scopesArray = $this->profile->getRequiredScope();
        return implode(',', $scopesArray);
    }

}

class JFBConnectProviderFacebookbusinessClient extends JFBCFacebook
{
    var $provider;

    public function isAuthenticated()
    {
        return (bool)$this->getUser();
    }

    function getToken()
    {
        return $this->getAccessToken();
    }

    // FB Business is server-side auth only
    public function authenticate()
    {
        $input = Factory::getApplication()->input;
        if ($input->getCmd('task') == 'login')
        {
            $params = array();
            $params['scope'] = JFBCFactory::provider('facebook')->getRequestScopeString();
            $params['redirect_uri'] = Uri::base() . 'index.php?option=com_jfbconnect&task=authenticate.callback&provider=facebookbusiness';
            $redirect = $this->getLoginUrl($params);
            Factory::getApplication()->redirect($redirect);
        }
        return;
    }
}